/*!\file flame.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Fichier contenant les entêtes des fonctions relatives à la structure Flame
 *
 */

#include "point.h"
#include <SDL/SDL.h>
#ifndef _flame_h_
#define _flame_h_

/*!\typedef Flame
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type Flame correspondant à la structure Flame
 *
 */

typedef struct Flame Flame;


/*!\struct Flame
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Structure désignant les fractales de type Flame
 *
 */

struct Flame{
	SDL_Surface* surf_screen;/*!<pointeur vers l'écran sur lequel on dessine les fractales de type Flamme*/
	double* pd_zoom;/*!<Un pointeur vers un réel représentant le zoom*/
	Point* ppoint_pInit;/*!<Un pointeur vers un point représentant le point initial de la fractale Flamme*/
	Point* ppoint_pitch;/*!<Un pointeur vers un point permettant de gérer le déplacement*/
	char* pc_fileName;/*!<Un chaine de caractère représentant le nom du fichier Flamme*/
	int i_nbPoint;/*!<Un entier représentant le nombre de points*/
	int i_maxColor;/*!<Un entier représentant la couleur du point le plus calculé*/
	int** ppi_tabColor;/*!<Un tableau deux dimensions d'entiers représentant des couleurs*/
	double* pd_color;/*!<Un pointeur vers un réel représentant une couleur*/
	int i_lockScreen;/*!<Un entier utilisé pour bloquer le redessinage de l'écran lors de l'augmentation du nombre de point*/
	double d_light;/*!<Un réel correspond à l'ajustement de l'intensité des couleurs*/
	int i_functionMode;/*!<Un entier correpondant au mode de coloration choisit par l'utilisateur, monochromatique ou coloration par fonction Ifs*/
	int** ppi_coloration;/*!<Un tableau d'entier représentant les différentes coloration*/
	int i_currentColor;/*!<Un entier représentant la couleur utilisée actuellement*/
};


/*!\fn Flame* initFlame(SDL_Surface* surf_screen, Point* ppoint_pInit, Point* ppoint_pitch, char* pc_fileName, double* pd_zoom, int i_nbPoint)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Crée une structure Flame, alloue la mémoire nécéssaire, et l'initialise
 *
 * \pre 
 * \post 
 *
 * \param surf_screen : Un pointeur vers un écran
 * \param ppoint_pInit : Un pointeur vers le point de départ de calcul de la fractale Flamme
 * \param ppoint_pitch : Un pointeur vers le point permettant de décaler la fractale Flamme
 * \param pc_fileName : Une chaine de caractère correspondant au nom du fichier Flamme
 * \param pd_zoom : Un pointeur vers le zoom
 * \param i_nbPoint : Entier représentant le nombre de points
 * \return Un pointeur vers une structure Flamme initialisée
 *
 */


Flame* initFlame(SDL_Surface *surf_screen, Point* ppoint_pInit, Point *ppoint_pitch, char* str_filePath, double *pd_zoom, int i_nbPoint);


/*!\fn void freeFlame(Flame* flame_flame)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Fonction libérant la mémoire allouée à la structure Flame et à ses attributs
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 *
 */

void freeFlame(Flame* flame_flame);


/*!\fn void createTabColorFlame(Flame* flame_flame)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Alloue la mémoire nécéssaire au tableau de couleurs Flame
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 *
 */



void createTabColorFlame(Flame* flame_flame);


/*!\fn void fillTabColorFlame(Flame* flame_flame, double* pd_tabProba, double** ppd_tabCoeff, char* pc_postfixX, char* pc_postfixY)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Rempli le tableau de couleurs Flame correspondant à la surface de l'écran
 * \remarks Pour chaque point Flame calculé, on incrémente une valeur représentant l'intensité de la couleur au niveau d'un pixel, ainsi plus le point est calculé, plus la couleur sera intense, on obtient ainsi un dégradé sur les fractales de type Flame. Les points les plus calculés sont colorés plus intensivement. Cette fonction est utilisée lorsque le calcul des transformations de type flamme n'est pas définit préalablement, c'est à dire que c'est l'utilisateur qui à rentré les transformations dans un fichier traité par l'analyseur syntaxique
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 * \param pd_tabProba : Un tableau de probabilité
 * \param ppd_tabCoeff : Un tableau deux dimensions de réels représentant des coefficients
 * \param pc_postfixX : Une chaine de caractère représentant la chaine postfixe transformée préalablement depuis la chaine infixe rentrée par l'utilisateur pour l'abscisse dans un fichier Flame
 * \param pc_postfixY : Une chaine de caractère représentant la chaine postfixe transformée préalablement depuis la chaine infixe rentrée par l'utilisateur pour l'ordonnée dans un fichier Flame
 *
 */

void fillTabColorFlame(Flame* flame_flame,double* pd_tabProba, double** ppd_tabCoeff, char* pc_postfixX, char* pc_postfixY);


/*!\fn void fillTabColorFlameChoice(Flame* flame_flame, double* pd_tabProba, double** ppd_tabCoeff, int i_flameNumber)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Rempli le tableau de couleurs Flame correspondant à la surface de l'écran
 * \remarks Pour chaque point Flame calculé, on incrémente une valeur représentant l'intensité de la couleur au niveau d'un pixel, ainsi plus le point est calculé, plus la couleur sera intense, on obtient ainsi un dégradé sur les fractales de type Flame. Les points les plus calculés sont colorés plus intensivement. Cette fonction est utilisée lorsque le calcul des transformations de type flamme est définit préalablement, c'est à dire que l'utilisateur à choisit parmis les flammes proposés dans le menu
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 * \param pd_tabProba : Un tableau de probabilité
 * \param ppd_tabCoeff : Un tableau deux dimensions de réels représentant des coefficients
 * \param i_flameNumber : Un entier correspondant au numéro de fractale flamme choisie par l'utilisateur
 *
 */

void fillTabColorFlameChoice(Flame* flame_flame,double* pd_tabProba, double** ppd_tabCoeff, int i_flameNumber);


/*!\fn SDL_Rect computingNewPointFlame(Flame* flame_flame, double* pd_tabCoef, char* pc_postfixX, char* pc_postfixY)
 *
 * \author Souplet Antoine
 * \version 1.0
 * \date 05/10/2011
 *
 * \brief Calcul les nouvelles coordonnées du point Flame à dessiner
 * \remarks Cette fonction est utilisée lorsque le calcul des transformations de type flamme n'est pas définit préalablement, c'est à dire que c'est l'utilisateur qui à rentré les transformations dans un fichier traité par l'analyseur syntaxique
 * 
 * \pre
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 * \param pd_tabCoef : Un tableau de double correspondant aux coefficient Ifs
  * \param pc_postfixX : Une chaine de caractère représentant la chaine postfixe transformée préalablement depuis la chaine infixe rentrée par l'utilisateur pour l'abscisse dans un fichier Flame
 * \param pc_postfixY : Une chaine de caractère représentant la chaine postfixe transformée préalablement depuis la chaine infixe rentrée par l'utilisateur pour l'ordonnée dans un fichier Flame
 *
 * \return un rectangle correspondant à la position du nouveau point à dessiner
 *
 */


SDL_Rect computingNewPointFlame(Flame* flame_flame, double* pd_tabCoef, char* pc_postfixX, char* pc_postfixY);


/*!\fn SDL_Rect computingNewPointFlameChoice(Flame* flame_flame, double* pd_tabCoef, int i_flameNumber)
 *
 * \author Souplet Antoine
 * \version 1.0
 * \date 05/10/2011
 *
 * \brief Calcul les nouvelles coordonnées du point Flame à dessiner
 * \remarks Cette fonction est utilisée lorsque le calcul des transformations de type flamme est définit préalablement, c'est à dire que l'utilisateur à choisit parmis les flammes proposés dans le menu
 * 
 * \pre
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 * \param pd_tabCoef : Un tableau de double correspondant aux coefficient Ifs
 * \param i_flameNumber : Un entier correspondant au numéro de fractale flamme choisie par l'utilisateur
 *
 * \return un rectangle correspondant à la position du nouveau point à dessiner
 *
 */


SDL_Rect computingNewPointFlameChoice(Flame* flame_flame, double* pd_tabCoef, int i_flameNumber);

/*!\fn int isChoiceFileName(char* pc_fileName)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Teste si l'utilisateur à choisit la flamme depuis le menu ou en ligne de commande 
 *
 * \pre 
 * \post 
 *
 * \param pc_fileName : Une chaine de caractère correspondant au nom du fichier Flame
 * \return Un entier, 1 si l'utilisateur à choisit parmis les flammes prédéfinies, 0 sinon 
 *
 */


int isChoiceFileName(char* pc_fileName);


/*!\fn int findIfsNumberFromChoice(char* pc_fileName)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Recherche le numéro de la fractale Ifs depuis une chaine de caractère
 *
 * \pre 
 * \post 
 *
 * \param pc_fileName : Une chaine de caractère représentant au nom du fichier Ifs
 * \return Un entier représentant le numéro associé à la fractale de type Ifs à dessiner
 *
 */

int findIfsNumberFromChoice(char* pc_fileName);


/*!\fn int findFlameNumberFromChoice(char* pc_fileName)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Recherche le numéro de la fractale flamme depuis une chaine de caractère
 *
 * \pre 
 * \post 
 *
 * \param pc_fileName : Une chaine de caractère représentant au nom du fichier Flame
 * \return Un entier représentant le numéro associé à la fractale de type flamme à dessiner
 *
 */

int findFlameNumberFromChoice(char* pc_fileName);


/*!\fn void testLockScreenFlame(Flame* flame_flame)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Bloque le redessinage de l'écran
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Un pointeur vers une structure Flame
 *
 */

void testLockScreenFlame(Flame* flame_flame);


/*!\fn Point calculFlame(double d_X, double d_Y, int i_flameNumber)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation lorsque la fonction de transformation est prédéfinie (pas d'utilisation de l'analyseur syntaxique)	
 *
 * \pre i_flameNumber correspond à un numéro de fracale flamme connue
 * \post 
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation associée à la fractale flamme
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation associée à la fractale flamme
 * \param i_flameNumber : Entier représentant le numéro de la fractale de type flamme à dessiner
 * \return Le point calculé après application de la transformation flamme 
 *
 */

Point calculFlame(double d_X, double d_Y, int i_flameNumber);


/*!\fn Point computeFlameV0(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV0(double d_X, double d_Y);

/*!\fn Point computeFlameV1(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 1
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 1
 * \return Le point calculé après application de la transformation flamme numéro 1
 *
 */

Point computeFlameV1(double d_X, double d_Y);

/*!\fn Point computeFlameV2(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 1
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 1
 * \return Le point calculé après application de la transformation flamme numéro 1
 *
 */

Point computeFlameV2(double d_X, double d_Y);

/*!\fn Point computeFlameV3(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 2
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 2
 * \return Le point calculé après application de la transformation flamme numéro 2
 *
 */

Point computeFlameV3(double d_X, double d_Y);

/*!\fn Point computeFlameV4(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 3
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 3
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 3
 * \return Le point calculé après application de la transformation flamme numéro 3
 *
 */

Point computeFlameV4(double d_X, double d_Y);

/*!\fn Point computeFlameV5(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 4
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 4
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 4
 * \return Le point calculé après application de la transformation flamme numéro 4
 *
 */

Point computeFlameV5(double d_X, double d_Y);

/*!\fn Point computeFlameV6(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 5	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 5
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 5
 * \return Le point calculé après application de la transformation flamme numéro 5
 *
 */

Point computeFlameV6(double d_X, double d_Y);

/*!\fn Point computeFlameV7(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 6
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 6
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 6
 * \return Le point calculé après application de la transformation flamme numéro 6
 *
 */

Point computeFlameV7(double d_X, double d_Y);

/*!\fn Point computeFlameV8(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV8(double d_X, double d_Y);

/*!\fn Point computeFlameV9(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV9(double d_X, double d_Y);

/*!\fn Point computeFlameV10(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV10(double d_X, double d_Y);

/*!\fn Point computeFlameV11(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV11(double d_X, double d_Y);

/*!\fn Point computeFlameV12(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV12(double d_X, double d_Y);

/*!\fn Point computeFlameV13(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV13(double d_X, double d_Y);


/*!\fn Point computeFlameV14(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV14(double d_X, double d_Y);


/*!\fn Point computeFlameV15(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV15(double d_X, double d_Y);

/*!\fn Point computeFlameV16(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV16(double d_X, double d_Y);

/*!\fn Point computeFlameV17(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV17(double d_X, double d_Y);

/*!\fn Point computeFlameV18(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV18(double d_X, double d_Y);

/*!\fn Point computeFlameV19(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV19(double d_X, double d_Y);

/*!\fn Point computeFlameV20(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV20(double d_X, double d_Y);


/*!\fn Point computeFlameV21(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV21(double d_X, double d_Y);


/*!\fn Point computeFlameV27(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV27(double d_X, double d_Y);

/*!\fn Point computeFlameV28(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV28(double d_X, double d_Y);

/*!\fn Point computeFlameV29(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV29(double d_X, double d_Y);

/*!\fn Point computeFlameV30(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV30(double d_X, double d_Y);

/*!\fn Point computeFlameV42(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV42(double d_X, double d_Y);


/*!\fn Point computeFlameV48(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV48(double d_X, double d_Y);


/*!\fn Point computeFlameV48(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV48(double d_X, double d_Y);


/*!\fn Point computeFlameV48(double d_X, double d_Y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 13-01-2012 
 *
 * \brief Calcul le point Flame après transformation 0	
 *
 * \param d_X : Abscisse du point sur lequel on applique la transformation flame numéro 0
 * \param d_Y : Ordonnéee du point sur lequel on applique la transformation flame numéro 0
 * \return Le point calculé après application de la transformation flamme numéro 0
 *
 */

Point computeFlameV48(double d_X, double d_Y);
#endif


