/*!\file ifs.h
 *
 * \author LEDIT Alexandre & SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Fichier contenant toutes les entêtes des fonctions associées à la fractale Ifs
 *
 */

#include "point.h"
#include <SDL/SDL.h>
#include "flame.h"
#ifndef _ifs_h_	
#define _ifs_h_

/*!\typedef Ifs
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type Ifs correspondant à la structure Ifs
 *
 */

typedef struct Ifs Ifs;


/*!\struct Ifs
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Structure représentant la fractale Ifs
 *
 */

struct Ifs {
	SDL_Surface *surf_screen;/*!<pointeur vers l'écran sur lequel on dessine les fractales de type Ifs*/
	double* pd_zoom;/*!<pointeur vers un double représentant le zoom*/
	Point* ppoint_pInit;/*!<pointeur vers un point représentant le point initial de calcul de la fractale Ifs*/
	Point* ppoint_pitch;/*!<pointeur vers un point permettant le décalage de la fractale Ifs pour le déplacement*/
	char* pc_fileName;/*!<chaine de caractère représentant le nom du fichier Ifs*/
	int i_nbPoint;/*!<entier correspondant au nombre de point à dessiner pour la fractale Ifs*/
	int i_maxColor;/*!<entier représentant la couleur du point le plus calculé*/
	int **ppi_tabColor;/*!<Un tableau à deux dimensions correspondant aux couleurs de dessin de la fractale Ifs*/
	double* pd_color;/*!<Un pointeur vers un réel correspondant à une couleur*/
	int i_lockScreen;/*!<Un entier bloquant le redessinage de l'écran lors de l'ajout de nouveaux points sur Ifs*/
	double d_light;/*!<Un réel représentant l'intensité de la couleur*/
	int i_functionMode;/*!<Un entier indiquant si on dessine en mode monochromatique ou en associant une couleur à chaque fonction Ifs*/
	int** ppi_coloration;/*!<Un tableau d'entier représentant les différentes coloration*/
	int i_currentColor;/*!<Un entier représentant la couleur utilisée actuellement*/
};


/*!\fn Ifs* initIfs(SDL_Surface* surf_screen, Point* ppoint_pInit, Point* ppoint_pitch, char* pc_fileName, double* pd_zoom, int i_nbPoint)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Crée et initialise une structure Ifs
 *
 * \param surf_screen : Un pointeur vers l'écran
 * \param ppoint_pInit : Un pointeur vers le point initial de calcul de la fractale Ifs
 * \param ppoint_pitch : Un pointeur vers le point permettant de gérer le déplacement
 * \param pc_fileName : Une chaine de caractère représentant le nom du fichier
 * \param pd_zoom : Un pointeur vers un réel représentant le zoom
 * \param i_nbPoint : Un entier représentant le nombre de point à calculer
 * \return une structure Ifs initialisée
 *
 */

Ifs* initIfs(SDL_Surface* surf_screen, Point* ppoint_pInit, Point* ppoint_pitch, char* pc_fileName, double* pd_zoom, int i_nbPoint);



/*!\fn void freeIfs(Ifs* ifs_ifs)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Fonction libérant la mémoire allouée à la structure Ifs et à ses attributs
 *
 * \param ifs_ifs : Un pointeur vers une structure Ifs
 *
 */

void freeIfs(Ifs* ifs_ifs);


/*!\fn int** initColoration()
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Initialise le tableau de coloration de l'écran pour Ifs et Flamme en association avec endColoration et beginColoration
 *
 * \return Un tableau à deux dimensions d'entier représentant tous les pixels de l'écran associés à une couleur
 *
 */


int** initColoration();


/*!\fn void beginColoration(int** ppi_coloration)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Initialise la coloration Ifs et Flamme en association avec endColoration et initColoration
 *
 * \pre 
 * \post 
 *
 * \param ppi_coloration : tableau à deux dimensions correpondant à la coloration de la fractale Ifs
 *
 */

void beginColoration(int **ppi_coloration);


/*!\fn void endColoration(int** ppi_coloration)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Initialise la coloration Ifs et Flamme en association avec beginColoration et initColoration
 *
 * \pre 
 * \post 
 *
 * \param ppi_coloration : tableau à deux dimensions correspondant à la coloration de la fractale Ifs
 *
 */

void endColoration(int **ppi_coloration);


/*!\fn void createTabColorIfs(Ifs* ifs_ifs)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Alloue la mémoire nécéssaire au tableau de couleurs Ifs
 *
 * \pre 
 * \post 
 *
 * \param ifs_ifs : Un pointeur vers une structure Ifs
 *
 */

void createTabColorIfs(Ifs* ifs_ifs);


/*!\fn void computingColor(double* pd_color, int** ppi_tabColor, int i_maxColor, SDL_Rect rect_position, double d_light, int i_functionMode, int** ppi_coloration, int i_currentColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief  Calcule la couleur de la fractale Ifs en fonction de paramètres tels que l'intensité lumineuse
 *
 * \pre 
 * \post 
 *
 * \param pd_color : Un pointeur vers une couleur représentant la couleur
 * \param ppi_tabColor : Un tableau à deux dimensions correspondant aux différentes couleurs Ifs
 * \param i_maxColor : Un entier représentant le nombre maximim de fois qu'un point à été calculé, soit le nombre de calcul correspond à la couleur maximale (afin de gérer le dégradé en fonction du nombre de calcul sur un point de l'écran)
 * \param rect_position : Un rectangle représentant la position du pixel à dessiner
 * \param d_light : Un réel représentant l'intensité des couleurs lors de dessin de la fractale Ifs ou flamme
 * \param i_functionMode : Un entier représentant le mode de coloration pour la fractale, soit monochromatique ou une couleur par fonction
 * \param ppi_coloration : Un tableau à deux dimmensions représentant le tableau d'entier représentant les valeurs de coloration pour chaque pixel de l'écran
 * \param i_currentColor : Un entier correspondant au numéro de la coloration actuelle
 *
 */

void computingColor(double* pd_color,int** ppi_tabColor, int i_maxColor, SDL_Rect rect_position, double d_light, int i_functionMode, int** ppi_coloration, int i_currentColor);



/*!\fn SDL_Rect computingNewPointIfs(Ifs* ifs_ifs, double* pd_tabCoef)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 05/10/2011
 *
 * \brief Calcul les nouvelles coordonnées du point Ifs à dessiner
 *
 * \pre i_functionNumber doit etre inférieur au nombre de fonction définit dans le fichier
 * \post 
 *
 * \param ifs_ifs : Un pointeur vers une structure Ifs
 * \param pd_tabCoef : Un tableau de double correspondant aux coefficient Ifs
 * \return un rectangle correspondant à la position du nouveau point à dessiner
 *
 */

SDL_Rect computingNewPointIfs(Ifs* ifs_ifs, double* pd_tabCoef);



/*!\fn double* getTabProba(double** ppd_tabCoef, int i_nbOfLine)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 27/10/2011
 *
 * \brief Fonction qui recupère les coefficient depuis le fichier IFS
 *
 * \pre 
 * \post 
 *
 * \param ppd_tabCoef : Un tableau de réel contenant les probabilité
 * \param i_nbOfLine : Un entier représentant le nombre de lignes dans le fichier
 * \return Un tableau de probabilité
 *
 */

double* getTabProba(double** ppd_tabCoef, int i_nbOfLine);


/*!\fn double** getCoeffTabFromFile(char* pc_filePath)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief fonction qui recupère une tableau de coefficients depuis un fichier Ifs
 *
 * \pre 
 * \post 
 *
 * \param pc_filePath : Une chaine de caractère représentant le nom du fichier Ifs
 * \return Un tableau deux dimensions de réels correspondant à des coefficients
 *
 */

double** getCoeffTabFromFile(char* str_filePath);


/*!\fn void fillTabColorIfs(Ifs* ifs_ifs, double* pd_tabProba, double** ppd_tabCoeff)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Rempli le tableau de couleurs Ifs correspondant à la surface de l'écran
 * \remarks Pour chaque point Ifs calculé, on incrémente une valeur représentant l'intensité de la couleur au niveau d'un pixel, ainsi plus le point est calculé, plus la couleur sera intense, on obtient ainsi un dégradé sur les fractales de type Ifs. Les points les plus calculés sont colorés plus intensivement
 *
 * \pre 
 * \post 
 *
 * \param ifs_ifs : Un pointeur vers une structure Ifs
 * \param pd_tabProba : Un tableau de probabilité
 * \param ppd_tabCoeff : Un tableau deux dimensions de réels représentant des coefficients
 *
 */

void fillTabColorIfs(Ifs* ifs_ifs,double* pd_tabProba, double** ppd_tabCoeff);


/*!\fn char* searchFileIfsChoice(int i_ifsNumber)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Retourne le nom de fichier Ifs choisit par l'utilisateur
 *
 * \pre 
 * \post 
 *
 * \param i_ifsNumber : Un entier représentant le numéro associée à une fractale
 * \return Une chaine de caractère correspondant au nom du fichier Ifs
 *
 */

char* searchFileIfsChoice(int i_ifsNumber);


/*!\fn void testLockScreenIfs(Ifs* ifs_ifs)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Bloque le redessinage de l'écran
 *
 * \pre 
 * \post 
 *
 * \param ifs_ifs : Un pointeur vers une structure Ifs
 *
 */

void testLockScreenIfs(Ifs* ifs_ifs);
#endif
