/*!\file lSystem.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Fichier contenant les entêtes des fonctions relatives à la structure L-System
 *
 */


#include "stack.h"
#include "point.h"
#include <stdio.h>
#include <SDL/SDL.h>
#include "functionFile.h"

#ifndef _lSystem_h_	
#define _lSystem_h_

/*!\def PI
 *
* \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définit une constante correspondant à la constante mathématique Pi
 *
 */

#define PI 3.14159265


/*!\def i_tabSize
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définit une constante correspondant à la taille d'un tableau
 *
 */


#define i_tabSize 10000000


/*!\typedef Lsystem
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type uCharDouble correspondant à la structure uCharDouble
 *
 */

typedef struct Lsystem Lsystem;


/*!\struct Lsystem
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Structure désignant les fractales de type L-System
 *
 */

struct Lsystem {
	SDL_Surface *surf_screen;/*!<pointeur vers l'écran sur lequel on dessine les fractales de type L-System*/
	double* pd_zoom;/*!<pointeur vers le zoom */
	Point* ppoint_pInit;/*!<pointeur vers le point initial de la fratale L-System*/
	char* pc_fileName;/*!<chaine de caractère correspondant au nom du fichier L-System*/
	char* pc_tabVariable;/*!<chaine de caractère regroupant la totalité des noms de variables utilisées pour une fractale L-System*/
	char* pc_tabFinal;/*!<chaine de caractère définissant la fractale L-System après calcul*/
	char** ppc_rules;/*!<tableau de chaine de caractère correspondant aux différentes règles*/
	int* pi_indice;/*!<pointeur vers un entier représentant l'indice auquel il faut dessiner la fractale L-System*/
	double d_angle;/*!<réel correspondant à l'angle définit pour une fractale de type L-System*/
	double d_longueur;/*!<!réél correspondant à l'angle définit pour une fractale de type L-System*/
};


/*!\fn Lsystem* initLsystem(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, char* pc_fileName, int* pi_indice)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Crée un pointeur vers une structure L-System, alloue la mémoire nécéssaire et initialise la structure
 *
 * \pre 
 * \post 
 *
 * \param surf_screen : Un pointeur vers l'écran sur lequel on dessine la fractale L-System
 * \param pd_zoom : Un pointeur vers un double correspondant au zoom
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point initial de calcul de la fractale L-System
 * \param pc_fileName : Une chaine de caractère correspondant au nom du fichier
 * \param pi_indice : Un pointeur vers un entier correspondant à l'indice pour lequel il faut dessiner la fractale de type L-System
 * \return Un pointeur vers la structure L-System initialisée
 *
 */

Lsystem* initLsystem(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, char* pc_fileName, int* pi_indice);


/*!\fn void freeLsystem(Lsystem* lsyst_lSystem)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Libère l'espace mémoire alloué à la structure lSystem et à ses attributs	
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure L-System
 *
 */

void freeLsystem(Lsystem* lsyst_lSystem);


/*!\fn void setLsystem(Lsystem* lsyst_lSystem, double d_angle, double d_longueur, char** ppc_rules, char* pc_tabVariable, char* pc_tabFinal)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Modifie les attributs de la structure L-System
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure de type L-System
 * \param d_angle : Un double correspondant à l'angle relatif à la transformation associée à la fractale L-System
 * \param d_longueur : Un double correspondant à la longueur relative à la transformation associée à la fractale L-System
 * \param ppc_rules : Un tableau de chaines de caractères contenant toutes les règles associée à la transformation de la fractale L-System
 * \param pc_tabVariable : Une chaine de caractère correspondant à un tableau de caractère représentant les différentes variables de la transformation L-System
 * \param pc_tabFinal : Une chaine de caractère correspondant au tableau final après calcul des transformations L-System 
 *
 */

void setLsystem(Lsystem* lsyst_lSystem, double d_angle, double d_longueur, char** ppc_rules, char* pc_tabVariable, char* pc_tabFinal);


/*!\fn void runLsystem(Lsystem* lsyst_lSystem)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Lance le traitement des fractales de type L-System
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure L-System correspondant à la fractale dont on lance le traitement
 *
 */

void runLsystem(Lsystem* lsyst_lSystem);


/*!\fn void drawFractalSystem(Lsystem* lsyst_lSystem)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Dessine les fractales de type L-System
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure L-System correspondant à la fractale que l'on dessine
 *
 */

void drawFractalSystem(Lsystem* lsyst_lSystem);


/*!\fn int checkCharAndSetParameters(Lsystem* lsyst_lSystem, int i_cpt, Point* point_pInit, Point* point_pSec, stack** st_saveCoordAndAngle, double* pd_angleCurrent)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Cherche le paramètre correspondant au caractère en cours de lecture et modifie l'angle courant ou les coordonnées des points en fonction du paramètre
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure L-System
 * \param i_cpt : Un entier représentant la position du caractère en cours de lecture
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point avant calcul
 * \param ppoint_pSec : Un pointeur vers un point correspond au point après calcul
 * \param st_saveCoordAndAngle : Un pointeur vers une pile permettant la sauvegarde des coordonnées d'un point et d'un angle
 * \param pd_angleCurrent : Un pointeur vers unréel correspondant à l'angle courant
 * \return Un entier, 1 si le caractère en cours de lecture à été traité, 0 sinon
 *
 */

int checkCharAndSetParameters(Lsystem* lsyst_lSystem, int i_cpt, Point* point_pInit, Point* point_pSec, stack** st_saveCoordAndAngle, double* pd_angleCurrent);


/*!\fn void pushInStackValues(stack** st_stack, Point* ppoint_point, double* pd_angleCurrent)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Empile les coordonnées du point et l'angle à sauvegarder dans une pile
 *
 * \pre 
 * \post 
 *
 * \param st_stack : Un pointeur vers une pile correspondant à la pile dans laquelle on sauvegarde les coordonnées du point ainsi que l'angle
 * \param ppoint_point : Un pointeur vers un point correspondant au point à sauvegarder dans la pile
 * \param pd_angleCurrent : Un pointeur vers un réel correspondant à l'angle à sauvegarder dans la pile
 *
 */

void pushInStackValues(stack** st_stack, Point* ppoint_point, double* pd_angleCurrent);


/*!\fn int checkRulesAndSetParameters(Lsystem* lsyst_lSystem, char c_value, Point* point_pInit, Point* point_pSec, double* pd_angleCurrent)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Recherche la règle correspondant au caractère en cours de lecture
 *
 * \pre 
 * \post 
 *
 * \param lsyst_lSystem : Un pointeur vers une structure L-System
 * \param c_value : Un caractère correspondant au caractère en cours de lecture
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point avant calcul
 * \param ppoint_pSec : Un pointeur vers un point correspond au point après calcul
 * \param pd_angleCurrent : Un pointeur vers un réel correspondant à l'angle courant
 * \return Un entier, 1 si le caractère en cours de lecture à été traité, 0 sinon
 *
 */

int checkRulesAndSetParameters(Lsystem* lsyst_lSystem, char c_value, Point* ppoint_pInit, Point* ppoint_pSec, double* pd_angleCurrent);


/*!\fn char* createFinalArray(char** ppc_rules, char* pc_tabVariable, int i_indice)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Crée le tableau de caractère correspondant à l'axiome et aux règles, que l'on développe jusqu'au rang passé en paramètre
 *
 * \pre 
 * \post 
 *
 * \param ppc_rules : Un tableau de chaine de caractère contenant toutes les règles relatives à la transformation L-System
 * \param pc_tabVariable : Une chaine de caractère représentant toutes les variables associées à la transformation L-System
 * \param i_indice : Un entier correspondant à l'indice auquel le tableau final de caractère doit être calculé
 * \return Une chaine de caractère correspondant au tableau de caractère final 
 *
 */

char* createFinalArray(char** ppc_rules, char* pc_tabVariable, int i_indice);


/*!\fn void concatCharToString(char c_value, char* pc_tabVariable, char* pc_tabFinal, char** ppc_rules);
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 11-01-2012 
 *
 * \brief Concatène le caractère passé en cours de lecture au tableau final
 *
 * \pre 
 * \post 
 *
 * \param c_value : Un caractère correspondant au caractère à concaténer au tableau final
 * \param pc_tabVariable : Une chaine de caractère représentant toutes les variables associées à la transformation L-System
 * \param pc_tabFinal : Une chaine de caractère correspondant au tableau de caractère final après traitement
 * \param ppc_rules : Un tableau de chaine de caractère contenant toutes les règles relatives à la transformation L-System
 * \return Une chaine de caractère correspondant au tableau de caractère final 
 *
 */

void concatCharToString(char c_value, char* pc_tabVariable, char* pc_tabFinal, char** ppc_rules);
#endif
