/*!\file mandelbrot.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Fichier contenant les entêtes de fonctions associées à la structure de Mandelbrot
 *
 */

#include "point.h"
#include <SDL/SDL.h>
#include <math.h>

#ifndef _mandelbrot_h_	
#define _mandelbrot_h_

/*!\typedef Mandelbrot
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type Mandelbrot correspondant à la structure Mandelbrot
 *
 */

typedef struct Mandelbrot Mandelbrot;


/*!\struct Mandelbrot
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Structure désignant la fractale de Mandelbrot
 *
 */

struct Mandelbrot{
	SDL_Surface *surf_screen;/*!<pointeur vers l'écran sur lequel on dessine la fractale de Mandelbrot*/
	double* pd_zoom;/*!<pointeur vers le zoom*/
	Point* ppoint_pInit;/*!<pointeur vers le point initial de la fractale de Mandelbrot*/
	int* pi_nbIter;/*!<pointeur vers le nombre d'itérations*/
	int* pi_numGradient;/*!<pointeur vers un entier représentant la coloration de la fractale de Mandelbrot*/
	int i_minX;/*!<abscisse à partir duquel on dessine Mandelbort*/
	int i_maxX;/*!<abscisse à partir duquel on arrête de dessiner Mandelbrot*/
	int i_minY;/*!<ordonnée à partir duquel on dessine Mandelbrot*/
	int i_maxY;/*!<ordonnée à partir duquel on arrête de dessiner Mandelbrot*/
};


/*!\fn initMandelbrot(SDL_Surface *surf_screen, double* pd_zoom, Point* ppoint_pInit, int i_minX, int i_maxX, int i_minY, int i_maxY, int* pi_nbIter, int* pi_numGradient)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Crée et initialise une structure Mandelbrot
 *
 * \pre 
 * \post 
 *
 * \param surf_screen : Un pointeur vers un écran
 * \param pd_zoom : Un pointeur vers un double correspondant au zoom
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point initial de la fractale de Mandelbrot
 * \param i_minX : Un entier définissant l'abscisse du point à partir duquel on dessine la fractale de Mandelbrot  
 * \param i_maxX : Un entier définissant l'abscisse du point jusqu'auquel on dessine la fractale de Mandelbrot 
 * \param i_minY : Un entier définissant l'ordonnée du point à partir duquel on dessine la fractale de Mandelbrot  
 * \param i_maxY : Un entier définissant l'ordonnée du point jusqu'auquel on dessine la fractale de Mandelbrot 
 * \param pi_nbIter : Un pointeur vers un entier correspondant au nombre d'iterations de la fractale de Mandebrot
 * \param pi_numGradient : Un pointeur vers un entier correspondant au choix de la coloration de la fractale de Mandelbrot
 *
 * \return Un pointeur vers une structure Mandelbrot 
 *
 */

Mandelbrot* initMandelbrot(SDL_Surface *surf_screen, double* pd_zoom, Point* ppoint_pInit, int i_minX, int i_maxX, int i_minY, int i_maxY, int* pi_nbIter, int* pi_numGradient);


/*!\fn void freeMandelbrot(Mandelbrot* mand_mandelbrot)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Libère l'espace mémoire alloué à une structure Mandelbrot et à ses attributs
 *
 * \pre 
 * \post 
 *
 * \param mand_mandelbrot : Un pointeur vers une structure Mandelbrot
 *
 */

void freeMandelbrot(Mandelbrot* mand_mandelbrot);



/*!\fn void freeThreadMandel(Mandelbrot* mand_mandel, int* pi_oldIter, int* pi_nIter)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Libère l'espace mémoire associé au traitement d'un thread de calcul de la fractale de Mandelbrot
 *
 * \pre 
 * \post 
 *
 * \param mand_mandel : Un pointeur vers une structure Mandelbrot
 * \param pi_oldIter : Un pointeur vers un entier correspondant au nombre d'itérations parcourues lors du calcul de la couleur du précédent pixel
 * \param pi_nIter : Un pointeur vers un entier correspondant au nombre d'itération parcourues lors du calcul du pixel en cours
 *
 */

void freeThreadMandel(Mandelbrot* mand_mandel, int* pi_oldIter, int* pi_nIter);


/*!\fn void freeLoopMandelbrotJulia(SDL_Surface* surf_pixel, int i_nbGradColor, double* pd_color, int** ppi_tabColor)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Libère l'espace mémoire alloué lors de la boucle de calcul de Mandelbrot ou de Julia
 *
 * \param surf_pixel : Un pointeur vers une structure Mandelbrot
 * \param i_nbGradColor : Un pointeur vers un entier correspondant au nombre d'itérations parcourues lors du calcul de la couleur du précédent pixel
 * \param pd_color : Un pointeur vers un entier correspondant au nombre d'itération parcourues lors du calcul du pixel en cours
 * \param ppi_tabColor : Un tableau deux dimensions représetnant les couleurs
 *
 */

void freeLoopMandelbrotJulia(SDL_Surface* surf_pixel, int i_nbGradColor, double* pd_color, int** ppi_tabColor);


/*!\fn int getNbGradColor(int i_numGradient)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Renvoie le nombre de couleur qui composent le dégradé
 *
 * \pre
 * \post
 *
 * \param i_numGradient : Un entier représentant le numéro du dégradé
 * 
 * \return Un entier correspondant au nombre de couleurs du dégradé
 *
 */

int getNbGradColor(int i_numGradient);


/*!\fn double* colorToRGB(double* pd_color, int** ppi_tabColor, int i_nbColor, int i_nbGradColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Modifie un tableau de couleur afin de correspondre au mieux aux fractales de Mandelbrot et de Julia
 *
 * \pre 
 * \post 
 *
 * \param pd_color : Un tableau de double contenant les valeurs RGB (red, green, blue) correspondantes à une couleur
 * \param ppi_tabColor : Un tableau à deux dimensions d'entier contenant des valeurs RGB (red, green, blue) correspondantes à des couleurs
 * \param i_nbColor : Un entier correspondant au nombre de couleurs presentes lors du dessin des fractales de Mandelbrot et de Julia 
 * \param i_nbGradColor : Un entier représentant le numéro du dégradé
 *
 * \return Un pointeur vers un double contenant les valeurs RGB (red, green, blue) correspondantes à une couleur 
 *
 */

double* colorToRGB(double* pd_color, int** ppi_tabColor, int i_nbColor, int i_nbGradColor);



/*!\fn int** createGrad1(int i_nbColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Crée le dégradé numéro 1
 *
 * \pre
 * \post
 *
 * \param i_nbColor : Un entier représentant le nombre de répétitions du dégradé dans la fractale
 * 
 *\return Un tableau correspondant aux couleurs du dégradé
 *
 */

int** createGrad1(int i_nbColor);


/*!\fn int** createGrad2(int i_nbColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Crée le dégradé numéro 2
 *
 * \pre
 * \post
 *
 * \param i_nbColor : Un entier représentant nombre de répétitions du dégradé dans la fractale
 * 
 *\return Un tableau correspondant aux couleurs du dégradé
 *
 */

int** createGrad2(int i_nbColor);


/*!\fn int** createGrad3(int i_nbColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Crée le dégradé numéro 3
 *
 * \pre
 * \post
 *
 * \param i_nbColor : Un entier représentant nombre de répétitions du dégradé dans la fractale
 * 
 *\return Un tableau correspondant aux couleurs du dégradé
 *
 */

int** createGrad3(int i_nbColor);


/*!\fn int** createGrad4(int i_nbColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Crée le dégradé numéro 4
 *
 * \pre
 * \post
 *
 * \param i_nbColor : Un entier représentant nombre de répétitions du dégradé dans la fractale
 * 
 *\return Un tableau correspondant aux couleurs du dégradé
 *
 */

int** createGrad4(int i_nbColor);


/*!\fn int** createGrad5(int i_nbColor)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 10-01-2012
 *
 * \brief Crée le dégradé numéro 5
 *
 * \pre
 * \post
 *
 * \param i_nbColor : Un entier représentant nombre de répétitions du dégradé dans la fractale
 * 
 *\return Un tableau correspondant aux couleurs du dégradé
 *
 */

int** createGrad5(int i_nbColor);



/*!\fn int** createTabColor(int i_numGradient)
 *
 * \author LEDIT Alexandre
 * \version 1.0
 * \date 08-01-2012 
 *
 * \brief Crée un tableau de couleurs en fonction du numéro du dégradé passé en paramètre
 *
 * \pre 
 * \post 
 *
 * \param i_numGradient : Entier correspondant au numéro du dégradé à sélectionner
 * \return Un tableau à deux dimmensions d'entiers correspondant à un tableau de couleurs
 */

int** createTabColor(int i_numGradient);


#endif
